/*
 * n01_view
 *
 * n01_view.c
 *
 * Copyright (C) 1996-2016 by Ohno Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>

#include "..\n01_plugin.h"
#include "window.h"
#include "score_window.h"
#include "result_window.h"
#include "Profile.h"

/* Define */
#define	INI_FILE_NAME				TEXT("n01_view.ini")

/* Global Variables */
static HINSTANCE hInst;
static HWND hViewWnd;
static HWND hScoreWnd;
static HWND hResultWnd;
static TCHAR ini_path[BUF_SIZE];

extern OPTION op;

/* Local Function Prototypes */

/*
 * get_color - F擾
 */
static COLORREF get_color(const TCHAR *buf, const COLORREF def_color)
{
	long hex;

	if (*buf == TEXT('\0')) {
		return def_color;
	}
	hex = _tcstol(buf, NULL, 0);
	return PALETTERGB(((BYTE)(hex >> 16) & 0xFF), ((BYTE)(hex >> 8) & 0xFF), ((BYTE)(hex) & 0xFF));
}

/*
 * put_color - F
 */
static void put_color(const TCHAR *ini_path, const TCHAR *group, const TCHAR *key, const COLORREF color)
{
	TCHAR buf[BUF_SIZE];

	wsprintf(buf, TEXT("0x%02X%02X%02X"), GetRValue(color), GetGValue(color), GetBValue(color));
	profile_write_string(group, key, buf, ini_path);
}

/*
 * DllMain - C
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
	TCHAR work_path[MAX_PATH];
	TCHAR buf[BUF_SIZE];
	TCHAR *p, *r;

	switch (fdwReason) {
	case DLL_PROCESS_ATTACH:
		hInst = hInstance;
		view_window_regist(hInst);
		score_window_regist(hInst);
		result_window_regist(hInst);

		// AvP[ṼpX擾
		GetModuleFileName(hInst, work_path, MAX_PATH - 1);
		for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
			if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
				p++;
				continue;
			}
#endif	// UNICODE
			if (*p == TEXT('\\') || *p == TEXT('/')) {
				r = p;
			}
		}
		*r = TEXT('\0');
		wsprintf(ini_path, TEXT("%s\\%s"), work_path, INI_FILE_NAME);

		profile_initialize(ini_path, TRUE);

		// EBhE
		SystemParametersInfo(SPI_GETWORKAREA, 0, &op.window_rect, 0);
		op.window_rect.left = profile_get_int(TEXT("view_window"), TEXT("left"), 0, ini_path);
		op.window_rect.top = profile_get_int(TEXT("view_window"), TEXT("top"), 0, ini_path);
		op.window_rect.right = profile_get_int(TEXT("view_window"), TEXT("right"), op.window_rect.right, ini_path);
		if (op.window_rect.right <= 10) {
			op.window_rect.right = 100;
		}
		op.window_rect.bottom = profile_get_int(TEXT("view_window"), TEXT("bottom"), op.window_rect.bottom, ini_path);
		if (op.window_rect.bottom <= 10) {
			op.window_rect.bottom = 100;
		}

		op.visible_set = profile_get_int(TEXT("view_window"), TEXT("visible_set"), 1, ini_path);
		op.size_width = profile_get_int(TEXT("view_window"), TEXT("size_width"), 3, ini_path);
		op.full_screen = profile_get_int(TEXT("view_window"), TEXT("full_screen"), 0, ini_path);

		// tHg
		profile_get_string(TEXT("view_window"), TEXT("font_name"), TEXT("Arial"), op.font_name, BUF_SIZE - 1, ini_path);
		op.name_font_size = profile_get_int(TEXT("view_window"), TEXT("name_font_size"), 0, ini_path);

		// F
		profile_get_string(TEXT("view_window"), TEXT("color_line_color"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.line_color = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("view_window"), TEXT("color_head_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.head_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("view_window"), TEXT("color_name_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.name_text = get_color(buf, RGB(255, 255, 0));
		profile_get_string(TEXT("view_window"), TEXT("color_name_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.name_back = get_color(buf, RGB(45, 45, 45));
		profile_get_string(TEXT("view_window"), TEXT("color_sets_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.sets_text = get_color(buf, RGB(255, 255, 0));
		profile_get_string(TEXT("view_window"), TEXT("color_sets_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.sets_back = get_color(buf, RGB(45, 45, 45));
		profile_get_string(TEXT("view_window"), TEXT("color_legs_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.legs_text = get_color(buf, RGB(255, 255, 0));
		profile_get_string(TEXT("view_window"), TEXT("color_legs_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.legs_back = get_color(buf, RGB(45, 45, 45));
		profile_get_string(TEXT("view_window"), TEXT("color_score_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_text = get_color(buf, RGB(255, 255, 0));
		profile_get_string(TEXT("view_window"), TEXT("color_score_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_back = get_color(buf, RGB(45, 45, 45));

		// 摜
		profile_get_string(TEXT("view_window"), TEXT("bmp_back"), TEXT(""), op.back_bmp, BUF_SIZE - 1, ini_path);
		profile_get_string(TEXT("view_window"), TEXT("bmp_num_l"), TEXT(""), op.num_l_bmp, BUF_SIZE - 1, ini_path);
		profile_get_string(TEXT("view_window"), TEXT("bmp_num_s"), TEXT(""), op.num_s_bmp, BUF_SIZE - 1, ini_path);

		op.name_left[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_left_1"), 0, ini_path);
		op.name_top[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_top_1"), 0, ini_path);
		op.name_right[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_right_1"), 0, ini_path);
		op.name_bottom[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_bottom_1"), 0, ini_path);
		op.score_x[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_score_x_1"), 0, ini_path);
		op.score_y[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_score_y_1"), 0, ini_path);
		op.sets_x[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_sets_x_1"), 0, ini_path);
		op.sets_y[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_sets_y_1"), 0, ini_path);
		op.legs_x[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_legs_x_1"), 0, ini_path);
		op.legs_y[0] = profile_get_int(TEXT("view_window"), TEXT("bmp_legs_y_1"), 0, ini_path);

		op.name_left[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_left_2"), 0, ini_path);
		op.name_top[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_top_2"), 0, ini_path);
		op.name_right[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_right_2"), 0, ini_path);
		op.name_bottom[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_name_bottom_2"), 0, ini_path);
		op.score_x[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_score_x_2"), 0, ini_path);
		op.score_y[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_score_y_2"), 0, ini_path);
		op.sets_x[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_sets_x_2"), 0, ini_path);
		op.sets_y[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_sets_y_2"), 0, ini_path);
		op.legs_x[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_legs_x_2"), 0, ini_path);
		op.legs_y[1] = profile_get_int(TEXT("view_window"), TEXT("bmp_legs_y_2"), 0, ini_path);

		// XRA\
		profile_get_string(TEXT("score_window"), TEXT("title"), TEXT(""), op.score_title, BUF_SIZE - 1, ini_path);

		op.score_window_rect.left = profile_get_int(TEXT("score_window"), TEXT("left"), 0, ini_path);
		op.score_window_rect.top = profile_get_int(TEXT("score_window"), TEXT("top"), 0, ini_path);
		op.score_window_rect.right = profile_get_int(TEXT("score_window"), TEXT("right"), 460, ini_path);
		if (op.score_window_rect.right <= 10) {
			op.score_window_rect.right = 460;
		}
		op.score_window_rect.bottom = profile_get_int(TEXT("score_window"), TEXT("bottom"), 114, ini_path);
		if (op.score_window_rect.bottom <= 10) {
			op.score_window_rect.bottom = 114;
		}

		op.score_visible_set = profile_get_int(TEXT("score_window"), TEXT("visible_set"), 1, ini_path);
		op.score_line_width = profile_get_int(TEXT("score_window"), TEXT("line_width"), 2, ini_path);
		op.score_size_width = profile_get_int(TEXT("score_window"), TEXT("size_width"), 3, ini_path);

		profile_get_string(TEXT("score_window"), TEXT("font_name"), TEXT("Arial"), op.score_font_name, BUF_SIZE - 1, ini_path);

		profile_get_string(TEXT("score_window"), TEXT("color_line_color"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_line_color = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("score_window"), TEXT("color_first_mark"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_first_mark = get_color(buf, RGB(0x80, 0, 0));

		profile_get_string(TEXT("score_window"), TEXT("color_head_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_head_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("score_window"), TEXT("color_head_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_head_back = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("score_window"), TEXT("color_name_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_name_text = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("score_window"), TEXT("color_name_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_name_back = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("score_window"), TEXT("color_sets_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_sets_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("score_window"), TEXT("color_sets_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_sets_back = get_color(buf, RGB(10, 36, 106));
		profile_get_string(TEXT("score_window"), TEXT("color_legs_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_legs_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("score_window"), TEXT("color_legs_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_legs_back = get_color(buf, RGB(10, 36, 106));
		profile_get_string(TEXT("score_window"), TEXT("color_score_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_score_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("score_window"), TEXT("color_score_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_score_back = get_color(buf, RGB(10, 36, 106));
		profile_get_string(TEXT("score_window"), TEXT("color_current_mark"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_current_mark = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("score_window"), TEXT("color_current_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.score_current_back = get_color(buf, RGB(255, 255, 255));

		// ʕ\
		op.reslut_rect.left = profile_get_int(TEXT("reslut_window"), TEXT("left"), 0, ini_path);
		op.reslut_rect.top = profile_get_int(TEXT("reslut_window"), TEXT("top"), 0, ini_path);
		op.reslut_rect.right = profile_get_int(TEXT("reslut_window"), TEXT("right"), 437, ini_path);
		if (op.reslut_rect.right <= 10) {
			op.reslut_rect.right = 437;
		}
		op.reslut_rect.bottom = profile_get_int(TEXT("reslut_window"), TEXT("bottom"), 357, ini_path);
		if (op.reslut_rect.bottom <= 10) {
			op.reslut_rect.bottom = 357;
		}
		op.reslut_line_width = profile_get_int(TEXT("reslut_window"), TEXT("reslut_line_width"), 2, ini_path);
		op.reslut_size_width = profile_get_int(TEXT("reslut_window"), TEXT("reslut_size_width"), 3, ini_path);
		op.reslut_full_screen = profile_get_int(TEXT("reslut_window"), TEXT("reslut_full_screen"), 0, ini_path);
		profile_get_string(TEXT("reslut_window"), TEXT("font_name"), TEXT("Arial"), op.reslut_font_name, BUF_SIZE - 1, ini_path);

		profile_get_string(TEXT("reslut_window"), TEXT("color_line_color"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_line_color = get_color(buf, RGB(0, 0, 0));
		profile_get_string(TEXT("reslut_window"), TEXT("color_head_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_head_back = get_color(buf, RGB(10, 36, 106));
		profile_get_string(TEXT("reslut_window"), TEXT("color_body_back"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_body_back = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("reslut_window"), TEXT("color_head_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_head_text = get_color(buf, RGB(255, 255, 255));
		profile_get_string(TEXT("reslut_window"), TEXT("color_title_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_title_text = get_color(buf, RGB(64, 64, 64));
		profile_get_string(TEXT("reslut_window"), TEXT("color_stats_text"), TEXT(""), buf, BUF_SIZE - 1, ini_path);
		op.reslut_stats_text = get_color(buf, RGB(0, 0, 0));

		profile_free();
		break;

	case DLL_PROCESS_DETACH:
		profile_initialize(ini_path, TRUE);

		// EBhE
		profile_write_int(TEXT("view_window"), TEXT("left"), op.window_rect.left, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("top"), op.window_rect.top, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("right"), op.window_rect.right, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bottom"), op.window_rect.bottom, ini_path);

		profile_write_int(TEXT("view_window"), TEXT("visible_set"), op.visible_set, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("size_width"), op.size_width, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("full_screen"), op.full_screen, ini_path);

		// tHg
		profile_write_string(TEXT("view_window"), TEXT("font_name"), op.font_name, ini_path);
		profile_write_int(TEXT("view_window"), TEXT("name_font_size"), op.name_font_size, ini_path);

		// F
		put_color(ini_path, TEXT("view_window"), TEXT("color_line_color"), op.line_color);
		put_color(ini_path, TEXT("view_window"), TEXT("color_head_text"), op.head_text);
		put_color(ini_path, TEXT("view_window"), TEXT("color_name_text"), op.name_text);
		put_color(ini_path, TEXT("view_window"), TEXT("color_name_back"), op.name_back);
		put_color(ini_path, TEXT("view_window"), TEXT("color_sets_text"), op.sets_text);
		put_color(ini_path, TEXT("view_window"), TEXT("color_sets_back"), op.sets_back);
		put_color(ini_path, TEXT("view_window"), TEXT("color_legs_text"), op.legs_text);
		put_color(ini_path, TEXT("view_window"), TEXT("color_legs_back"), op.legs_back);
		put_color(ini_path, TEXT("view_window"), TEXT("color_score_text"), op.score_text);
		put_color(ini_path, TEXT("view_window"), TEXT("color_score_back"), op.score_back);

		// 摜
		profile_write_string(TEXT("view_window"), TEXT("bmp_back"), op.back_bmp, ini_path);
		profile_write_string(TEXT("view_window"), TEXT("bmp_num_l"), op.num_l_bmp, ini_path);
		profile_write_string(TEXT("view_window"), TEXT("bmp_num_s"), op.num_s_bmp, ini_path);

		profile_write_int(TEXT("view_window"), TEXT("bmp_name_left_1"), op.name_left[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_top_1"), op.name_top[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_right_1"), op.name_right[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_bottom_1"), op.name_bottom[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_score_x_1"), op.score_x[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_score_y_1"), op.score_y[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_sets_x_1"), op.sets_x[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_sets_y_1"), op.sets_y[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_legs_x_1"), op.legs_x[0], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_legs_y_1"), op.legs_y[0], ini_path);

		profile_write_int(TEXT("view_window"), TEXT("bmp_name_left_2"), op.name_left[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_top_2"), op.name_top[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_right_2"), op.name_right[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_name_bottom_2"), op.name_bottom[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_score_x_2"), op.score_x[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_score_y_2"), op.score_y[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_sets_x_2"), op.sets_x[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_sets_y_2"), op.sets_y[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_legs_x_2"), op.legs_x[1], ini_path);
		profile_write_int(TEXT("view_window"), TEXT("bmp_legs_y_2"), op.legs_y[1], ini_path);


		// XRA\
		profile_write_string(TEXT("score_window"), TEXT("title"), op.score_title, ini_path);

		profile_write_int(TEXT("score_window"), TEXT("left"), op.score_window_rect.left, ini_path);
		profile_write_int(TEXT("score_window"), TEXT("top"), op.score_window_rect.top, ini_path);
		profile_write_int(TEXT("score_window"), TEXT("right"), op.score_window_rect.right, ini_path);
		profile_write_int(TEXT("score_window"), TEXT("bottom"), op.score_window_rect.bottom, ini_path);

		profile_write_int(TEXT("score_window"), TEXT("visible_set"), op.score_visible_set, ini_path);
		profile_write_int(TEXT("score_window"), TEXT("line_width"), op.score_line_width, ini_path);
		profile_write_int(TEXT("score_window"), TEXT("size_width"), op.score_size_width, ini_path);

		profile_write_string(TEXT("score_window"), TEXT("font_name"), op.score_font_name, ini_path);

		put_color(ini_path, TEXT("score_window"), TEXT("color_line_color"), op.score_line_color);
		put_color(ini_path, TEXT("score_window"), TEXT("color_first_mark"), op.score_first_mark);
		put_color(ini_path, TEXT("score_window"), TEXT("color_head_text"), op.score_head_text);
		put_color(ini_path, TEXT("score_window"), TEXT("color_head_back"), op.score_head_back);
		put_color(ini_path, TEXT("score_window"), TEXT("color_name_text"), op.score_name_text);
		put_color(ini_path, TEXT("score_window"), TEXT("color_name_back"), op.score_name_back);
		put_color(ini_path, TEXT("score_window"), TEXT("color_sets_text"), op.score_sets_text);
		put_color(ini_path, TEXT("score_window"), TEXT("color_sets_back"), op.score_sets_back);
		put_color(ini_path, TEXT("score_window"), TEXT("color_legs_text"), op.score_legs_text);
		put_color(ini_path, TEXT("score_window"), TEXT("color_legs_back"), op.score_legs_back);
		put_color(ini_path, TEXT("score_window"), TEXT("color_score_text"), op.score_score_text);
		put_color(ini_path, TEXT("score_window"), TEXT("color_score_back"), op.score_score_back);
		put_color(ini_path, TEXT("score_window"), TEXT("color_current_mark"), op.score_current_mark);
		put_color(ini_path, TEXT("score_window"), TEXT("color_current_back"), op.score_current_back);

		// ʕ\
		profile_write_int(TEXT("reslut_window"), TEXT("left"), op.reslut_rect.left, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("top"), op.reslut_rect.top, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("right"), op.reslut_rect.right, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("bottom"), op.reslut_rect.bottom, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("reslut_line_width"), op.reslut_line_width, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("reslut_size_width"), op.reslut_size_width, ini_path);
		profile_write_int(TEXT("reslut_window"), TEXT("reslut_full_screen"), op.reslut_full_screen, ini_path);
		profile_write_string(TEXT("reslut_window"), TEXT("font_name"), op.reslut_font_name, ini_path);

		put_color(ini_path, TEXT("reslut_window"), TEXT("color_line_color"), op.reslut_line_color);
		put_color(ini_path, TEXT("reslut_window"), TEXT("color_head_back"), op.reslut_head_back);
		put_color(ini_path, TEXT("reslut_window"), TEXT("color_body_back"), op.reslut_body_back);
		put_color(ini_path, TEXT("reslut_window"), TEXT("color_head_text"), op.reslut_head_text);
		put_color(ini_path, TEXT("reslut_window"), TEXT("color_title_text"), op.reslut_title_text);
		put_color(ini_path, TEXT("reslut_window"), TEXT("color_stats_text"), op.reslut_stats_text);

		profile_flush(ini_path);
		profile_free();
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
	return TRUE;
}

/*
 * get_plugin_info - vOC擾
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		index - 擾̃CfbNX (0`)
 *		pgi - vOC擾
 *
 *	߂l:
 *		TRUE - Ɏ擾vOC
 *		FALSE - 擾̏I
 */
__declspec(dllexport) BOOL CALLBACK get_plugin_info(const HWND hWnd, const int index, PLUGIN_GET_INFO *pgi)
{
	WORD lang = PRIMARYLANGID(LANGIDFROMLCID(GetThreadLocale()));

	switch (index) {
	case 0:
		if (lang != LANG_JAPANESE) {
			lstrcpy(pgi->title, TEXT("Score View (&Large)"));
		} else {
			lstrcpy(pgi->title, TEXT("傫XRA\(&V)"));
		}
		lstrcpy(pgi->func_name, TEXT("n01_view"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;

	case 1:
		if (lang != LANG_JAPANESE) {
			lstrcpy(pgi->title, TEXT("Score View (&Small)"));
		} else {
			lstrcpy(pgi->title, TEXT("XRA\(&S)"));
		}
		lstrcpy(pgi->func_name, TEXT("n01_socre"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;

	case 2:
		if (lang != LANG_JAPANESE) {
			lstrcpy(pgi->title, TEXT("&Result View"));
		} else {
			lstrcpy(pgi->title, TEXT("ʂ\(&R)"));
		}
		lstrcpy(pgi->func_name, TEXT("n01_result"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;

	case 3:
		return FALSE;
	}
	return FALSE;
}

/*
 * func_plugin_initialize - vOC̏
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		vOČĂяo@
 *
 *	l:
 *		֐ vOC_initialize
 */
__declspec(dllexport) BOOL CALLBACK n01_view_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// ߂lɌĂяo@̑gݍ킹ݒ肷
	return (CALLTYPE_END | CALLTYPE_MENU | CALLTYPE_INPUT_START | CALLTYPE_FINISH);
}

/*
 * func_plugin_property - vOC̃vpeB
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		PLUGIN_
 *
 *	l:
 *		֐ vOC_property
 *		ݒ肷vpeBꍇÅ֐͕KvB
 */
__declspec(dllexport) BOOL CALLBACK n01_view_property(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// vpeBݒ肵ꍇ PLUGIN_SUCCEED Ԃ
	// ݒ肷vpeBꍇ PLUGIN_ERROR Ԃ
	return PLUGIN_ERROR;
}

/*
 * func_plugin - vOC
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		call_type - Ăяo@
 *		pei - vOCs
 *		param - Ăяo
 *
 *	߂l:
 *		PLUGIN_
 */
__declspec(dllexport) int CALLBACK n01_view(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	switch (call_type) {
	case CALLTYPE_END:
		if (hViewWnd != NULL && IsWindow(hViewWnd) == TRUE) {
			SendMessage(hViewWnd, WM_CLOSE, 0, 0);
			hViewWnd = NULL;
		}
		break;

	case CALLTYPE_MENU:
		if (hViewWnd == NULL || IsWindow(hViewWnd) != TRUE) {
			hViewWnd = view_window_create(hInst, pei->si);
		} else {
			SetForegroundWindow(hViewWnd);
		}
		break;

	case CALLTYPE_INPUT_START:
		if (hViewWnd == NULL || IsWindow(hViewWnd) != TRUE) {
			hViewWnd = NULL;
		} else {
			SendMessage(hViewWnd, WM_SCORE_REDRAW, 0, 0);
		}
		break;

	case CALLTYPE_FINISH:
		if (hViewWnd == NULL || IsWindow(hViewWnd) != TRUE) {
			hViewWnd = NULL;
		} else {
			SendMessage(hViewWnd, WM_SCORE_REDRAW_DELAY, 0, 0);
		}
		break;
	}
	return PLUGIN_SUCCEED;
}

/*
 * func_plugin_initialize - vOC̏
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		vOČĂяo@
 *
 *	l:
 *		֐ vOC_initialize
 */
__declspec(dllexport) BOOL CALLBACK n01_socre_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// ߂lɌĂяo@̑gݍ킹ݒ肷
	return (CALLTYPE_END | CALLTYPE_MENU | CALLTYPE_INPUT_START | CALLTYPE_FINISH);
}

/*
 * func_plugin_property - vOC̃vpeB
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		PLUGIN_
 *
 *	l:
 *		֐ vOC_property
 *		ݒ肷vpeBꍇÅ֐͕KvB
 */
__declspec(dllexport) BOOL CALLBACK n01_socre_property(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// vpeBݒ肵ꍇ PLUGIN_SUCCEED Ԃ
	// ݒ肷vpeBꍇ PLUGIN_ERROR Ԃ
	return PLUGIN_ERROR;
}

/*
 * func_plugin - vOC
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		call_type - Ăяo@
 *		pei - vOCs
 *		param - Ăяo
 *
 *	߂l:
 *		PLUGIN_
 */
__declspec(dllexport) int CALLBACK n01_socre(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	switch (call_type) {
	case CALLTYPE_END:
		if (hScoreWnd != NULL && IsWindow(hScoreWnd) == TRUE) {
			SendMessage(hScoreWnd, WM_CLOSE, 0, 0);
			hScoreWnd = NULL;
		}
		break;

	case CALLTYPE_MENU:
		if (hScoreWnd == NULL || IsWindow(hScoreWnd) != TRUE) {
			hScoreWnd = score_window_create(hInst, pei->si);
		} else {
			SetForegroundWindow(hScoreWnd);
		}
		break;

	case CALLTYPE_INPUT_START:
		if (hScoreWnd == NULL || IsWindow(hScoreWnd) != TRUE) {
			hScoreWnd = NULL;
		} else {
			SendMessage(hScoreWnd, WM_SCORE_REDRAW, 0, 0);
		}
		break;

	case CALLTYPE_FINISH:
		if (hScoreWnd == NULL || IsWindow(hScoreWnd) != TRUE) {
			hScoreWnd = NULL;
		} else {
			SendMessage(hScoreWnd, WM_SCORE_REDRAW_DELAY, 0, 0);
		}
		break;
	}
	return PLUGIN_SUCCEED;
}

/*
 * func_plugin_initialize - vOC̏
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		vOČĂяo@
 *
 *	l:
 *		֐ vOC_initialize
 */
__declspec(dllexport) BOOL CALLBACK n01_result_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// ߂lɌĂяo@̑gݍ킹ݒ肷
	return (CALLTYPE_END | CALLTYPE_MENU | CALLTYPE_LEG_START | CALLTYPE_INPUT_START | CALLTYPE_FINISH);
}

/*
 * func_plugin_property - vOC̃vpeB
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		PLUGIN_
 *
 *	l:
 *		֐ vOC_property
 *		ݒ肷vpeBꍇÅ֐͕KvB
 */
__declspec(dllexport) BOOL CALLBACK n01_result_property(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// vpeBݒ肵ꍇ PLUGIN_SUCCEED Ԃ
	// ݒ肷vpeBꍇ PLUGIN_ERROR Ԃ
	return PLUGIN_ERROR;
}

/*
 * func_plugin - vOC
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		call_type - Ăяo@
 *		pei - vOCs
 *		param - Ăяo
 *
 *	߂l:
 *		PLUGIN_
 */
__declspec(dllexport) int CALLBACK n01_result(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	switch (call_type) {
	case CALLTYPE_END:
		if (hResultWnd != NULL && IsWindow(hResultWnd) == TRUE) {
			SendMessage(hResultWnd, WM_CLOSE, 0, 0);
			hResultWnd = NULL;
		}
		break;

	case CALLTYPE_MENU:
		if (hResultWnd == NULL || IsWindow(hResultWnd) != TRUE) {
			OPTION_INFO *opi = (OPTION_INFO *)SendMessage(hWnd, WM_N01_GET_OPTION, 0, 0);
			op.avg_per_round = opi->opi.avg_per_round;
			hResultWnd = result_window_create(hInst, pei->si);
		} else {
			SendMessage(hResultWnd, WM_CLOSE, 0, 0);
			hResultWnd = NULL;
		}
		break;

	case CALLTYPE_LEG_START:
	case CALLTYPE_INPUT_START:
	case CALLTYPE_FINISH:
		if (hResultWnd == NULL || IsWindow(hResultWnd) != TRUE) {
			hResultWnd = NULL;
		} else {
			SendMessage(hResultWnd, WM_SCORE_REDRAW, 0, 0);
		}
		break;
	}
	return PLUGIN_SUCCEED;
}
/* End of source */
