/*
 * n01_score
 *
 * result.c
 *
 * Copyright (C) 1996-2007 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "Memory.h"
#include "String.h"
#include "font.h"
#include "window.h"
#include "result_window.h"

//#include "resource.h"

/* Define */
#define RESLUT_WINDOW_CLASS			TEXT("n01_result_wnd")
#define RESLUT_WINDOW_TITLE			TEXT("n01 Result")

#define DRAW_LEFT					0
#define DRAW_RIGHT					1
#define DRAW_CENTER					2

/* Global Variables */
extern HINSTANCE hInst;

OPTION op;

typedef struct _DRAW_BUFFER {
	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;

	HFONT name_font;
	HFONT score_font;
	int font_size;

	HBRUSH back_brush;
	HBRUSH header_back_brush;
	HBRUSH body_back_brush;

	SCORE_INFO *si;
} DRAW_BUFFER;

/* Local Function Prototypes */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_result(const HWND hWnd, const DRAW_BUFFER *bf);
static LRESULT CALLBACK result_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * draw_init - `̏
 */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf)
{
	HDC hdc;
	RECT rect;

	GetClientRect(hWnd, &rect);

	hdc = GetDC(hWnd);
	bf->draw_bmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
	bf->draw_ret_bmp = SelectObject(bf->draw_dc, bf->draw_bmp);

	bf->font_size = ((rect.right < rect.bottom) ? rect.right : rect.bottom);
	while (1) {
		HFONT tmp_font;
		HFONT ret_font;
		TEXTMETRIC tm;
		int font_height;

		tmp_font = font_create(op.reslut_font_name, bf->font_size, FW_BOLD, FALSE, FALSE);
		ret_font = SelectObject(hdc, tmp_font);
		GetTextMetrics(hdc, &tm);
		font_height = tm.tmHeight;
		SelectObject(hdc, ret_font);
		if (tmp_font != NULL) {
			DeleteObject(tmp_font);
		}
		if (((font_height + tm.tmHeight / 6) * 11 - op.reslut_line_width * 2) < rect.bottom) {
			break;
		}
		bf->font_size--;
	}
	ReleaseDC(hWnd, hdc);

	bf->name_font = font_create_menu(bf->font_size, FW_BOLD, FALSE);
	bf->score_font = font_create(op.reslut_font_name, bf->font_size, FW_BOLD, FALSE, FALSE);
	return TRUE;
}

/*
 * draw_free - `̉
 */
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf)
{
	if (bf->draw_dc != NULL) {
		SelectObject(bf->draw_dc, bf->draw_ret_bmp);
		DeleteObject(bf->draw_bmp);
		bf->draw_bmp = NULL;
	}
	if (bf->name_font != NULL) {
		DeleteObject(bf->name_font);
		bf->name_font = NULL;
	}
	if (bf->score_font != NULL) {
		DeleteObject(bf->score_font);
		bf->score_font = NULL;
	}
	return TRUE;
}

/*
 * draw_text - eLXg̕`
 */
static void draw_text(const HDC hdc, const TCHAR *str, const int len, const RECT *rect, const int type)
{
	SIZE sz;
	int left, top;

	GetTextExtentPoint32(hdc, str, len, &sz);
	top = rect->top + (rect->bottom - rect->top - sz.cy) / 2;
	switch (type) {
	case DRAW_CENTER:
		left = rect->left + (rect->right - rect->left - sz.cx) / 2;
		break;

	case DRAW_RIGHT:
		left = rect->right - sz.cx;
		break;

	case DRAW_LEFT:
	default:
		left = rect->left;
		break;
	}
	ExtTextOut(hdc, left, top, ETO_CLIPPED, rect, str, len, NULL);
}

/*
 * draw_result - ʂ̕`
 */
static BOOL draw_result(const HWND hWnd, const DRAW_BUFFER *bf)
{
	RECT rect;
	RECT head_rect;
	RECT body_rect;
	RECT tmp_rect;
	HFONT ret_font;
	TEXTMETRIC tm;
	TCHAR buf[BUF_SIZE];
	int font_width;
	int font_height;
	int i, j;
	int top;
	int height;
	int type;

	GetClientRect(hWnd, &rect);
	FillRect(bf->draw_dc, &rect, bf->back_brush);

	rect.left += op.reslut_line_width;
	rect.top += op.reslut_line_width;
	rect.right -= op.reslut_line_width;
	rect.bottom -= op.reslut_line_width;

	ret_font = SelectObject(bf->draw_dc, bf->score_font);
	GetTextMetrics(bf->draw_dc, &tm);
	font_width = tm.tmAveCharWidth * 6;
	font_height = tm.tmHeight + tm.tmHeight / 6;
	SelectObject(bf->draw_dc, ret_font);

	// Header
	SetRect(&head_rect, rect.left, rect.top, rect.right, font_height);
	FillRect(bf->draw_dc, &head_rect, bf->header_back_brush);

	// Body
	top = head_rect.bottom + op.reslut_line_width;
	SetRect(&body_rect, rect.left, top, rect.right, rect.bottom);
	FillRect(bf->draw_dc, &body_rect, bf->body_back_brush);

	SetRect(&head_rect, rect.left + tm.tmAveCharWidth, rect.top, rect.right - tm.tmAveCharWidth, font_height);
	SetRect(&body_rect, rect.left + tm.tmAveCharWidth, top, rect.right - tm.tmAveCharWidth, rect.bottom);

	type = DRAW_LEFT;
	for (i = 0; i < 2; i++) {
		RECT name_draw_rect;
		int head_width = head_rect.right - head_rect.left;
		// name
		ret_font = SelectObject(bf->draw_dc, bf->name_font);
		SetTextColor(bf->draw_dc, op.reslut_head_text);
		if (type == DRAW_LEFT) {
			SetRect(&name_draw_rect, head_rect.left, head_rect.top, head_rect.left + head_width / 2, head_rect.bottom);
			DrawText(bf->draw_dc, bf->si->player[i].name, lstrlen(bf->si->player[i].name), &name_draw_rect,
				DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		} else {
			SetRect(&name_draw_rect, head_rect.left + head_width / 2, head_rect.top, head_rect.right, head_rect.bottom);
			DrawText(bf->draw_dc, bf->si->player[i].name, lstrlen(bf->si->player[i].name), &name_draw_rect,
				DT_RIGHT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		}
		SelectObject(bf->draw_dc, ret_font);

		height = body_rect.top;
		ret_font = SelectObject(bf->draw_dc, bf->score_font);

		// Set
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("Sets"), lstrlen(TEXT("Sets")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot(bf->si->player[i].sets, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// Leg
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("Legs"), lstrlen(TEXT("Legs")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot(bf->si->player[i].legs, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;
		
		// 100+
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("100+"), lstrlen(TEXT("100+")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot(bf->si->player[i].stat.ton00_count + bf->si->player[i].set_stat.ton00_count, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// 140+
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("140+"), lstrlen(TEXT("140+")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot(bf->si->player[i].stat.ton40_count + bf->si->player[i].set_stat.ton40_count, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// 180's
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("180's"), lstrlen(TEXT("180's")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot(bf->si->player[i].stat.ton80_count + bf->si->player[i].set_stat.ton80_count, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// High Out
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("High Out"), lstrlen(TEXT("High Out")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		_itot((bf->si->player[i].stat.high_off > bf->si->player[i].set_stat.high_off) ?
			bf->si->player[i].stat.high_off : bf->si->player[i].set_stat.high_off, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// Best Leg
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("Best Leg"), lstrlen(TEXT("Best Leg")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		if (bf->si->player[i].stat.short_game == 0 && bf->si->player[i].set_stat.short_game > 0) {
			j = bf->si->player[i].set_stat.short_game;
		} else if (bf->si->player[i].set_stat.short_game == 0 && bf->si->player[i].stat.short_game > 0) {
			j = bf->si->player[i].stat.short_game;
		} else {
			j = (bf->si->player[i].stat.short_game < bf->si->player[i].set_stat.short_game) ? bf->si->player[i].stat.short_game : bf->si->player[i].set_stat.short_game;
		}
		_itot(j, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// Darts Ave
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("Darts"), lstrlen(TEXT("Darts")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		if (bf->si->player[i].stat.win_count + bf->si->player[i].set_stat.win_count > 0) {
			_stprintf(buf, TEXT("%.2f"),
				(float)(bf->si->player[i].stat.win_darts + bf->si->player[i].set_stat.win_darts) / (float)(bf->si->player[i].stat.win_count + bf->si->player[i].set_stat.win_count));
		} else {
			lstrcpy(buf, TEXT("0.00"));
		}
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// Score Ave
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("Score"), lstrlen(TEXT("Score")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		if (bf->si->player[i].stat.all_darts + bf->si->player[i].set_stat.all_darts > 0) {
			if (op.avg_per_round == 1) {
				_stprintf(buf, TEXT("%.2f"),
					(float)(bf->si->player[i].stat.all_score + bf->si->player[i].set_stat.all_score) / (float)(bf->si->player[i].stat.all_darts + bf->si->player[i].set_stat.all_darts) * 3);
			} else {
				_stprintf(buf, TEXT("%.2f"),
					(float)(bf->si->player[i].stat.all_score + bf->si->player[i].set_stat.all_score) / (float)(bf->si->player[i].stat.all_darts + bf->si->player[i].set_stat.all_darts));
			}
		} else {
			lstrcpy(buf, TEXT("0.00"));
		}
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;

		// First 9 darts
		if (i == 0) {
			SetRect(&tmp_rect, rect.left, height, rect.right, height + font_height);
			SetTextColor(bf->draw_dc, op.reslut_title_text);
			draw_text(bf->draw_dc, TEXT("First 9"), lstrlen(TEXT("First 9")), &tmp_rect, DRAW_CENTER);

			SetRect(&tmp_rect, rect.left, height, rect.right / 2 - font_width, height + font_height);
		} else {
			SetRect(&tmp_rect, rect.right / 2 + font_width, height, rect.right, height + font_height);
		}
		SetTextColor(bf->draw_dc, op.reslut_stats_text);
		if (bf->si->player[i].stat.first9_darts + bf->si->player[i].set_stat.first9_darts > 0) {
			if (op.avg_per_round == 1) {
				_stprintf(buf, TEXT("%.2f"),
					(float)(bf->si->player[i].stat.first9_score + bf->si->player[i].set_stat.first9_score) / (float)(bf->si->player[i].stat.first9_darts + bf->si->player[i].set_stat.first9_darts) * 3);
			} else {
				_stprintf(buf, TEXT("%.2f"),
					(float)(bf->si->player[i].stat.first9_score + bf->si->player[i].set_stat.first9_score) / (float)(bf->si->player[i].stat.first9_darts + bf->si->player[i].set_stat.first9_darts));
			}
		} else {
			lstrcpy(buf, TEXT("0.00"));
		}
		draw_text(bf->draw_dc, buf, lstrlen(buf), &tmp_rect, DRAW_CENTER);
		height += font_height;
		SelectObject(bf->draw_dc, ret_font);

		type = DRAW_RIGHT;
	}
	return TRUE;
}

/*
 * result_window_proc - ʕ\EBhEvV[W
 */
static LRESULT CALLBACK result_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	DRAW_BUFFER *bf;
	SCORE_INFO *si;
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;

	switch (msg) {
	case WM_CREATE:
		si = (SCORE_INFO *)((CREATESTRUCT *)lParam)->lpCreateParams;
		if (si == NULL) {
			return -1;
		}
		bf = (DRAW_BUFFER *)mem_calloc(sizeof(DRAW_BUFFER));
		if (bf == NULL) {
			return -1;
		}
		bf->si = si;

		// `p
		hdc = GetDC(hWnd);
		bf->draw_dc = CreateCompatibleDC(hdc);
		ReleaseDC(hWnd, hdc);
		draw_init(hWnd, bf);
		SetBkMode(bf->draw_dc, TRANSPARENT);
		bf->back_brush = CreateSolidBrush(op.reslut_line_color);
		bf->header_back_brush = CreateSolidBrush(op.reslut_head_back);
		bf->body_back_brush = CreateSolidBrush(op.reslut_body_back);
		draw_result(hWnd, bf);

		SetWindowLong(hWnd, GWL_USERDATA, (LONG)bf);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			draw_free(hWnd, bf);
			if (bf->draw_dc != NULL) {
				DeleteDC(bf->draw_dc);
				bf->draw_dc = NULL;
			}
			DeleteObject(bf->back_brush);
			DeleteObject(bf->header_back_brush);
			DeleteObject(bf->body_back_brush);
			mem_free(&bf);
		}
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SIZE:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		draw_result(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_EXITSIZEMOVE:
		if (op.reslut_full_screen != 0) {
			break;
		}
		// TCYύX
		if (IsWindowVisible(hWnd) != 0 && IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
			GetWindowRect(hWnd, (LPRECT)&op.reslut_rect);
			op.reslut_rect.right -= op.reslut_rect.left;
			op.reslut_rect.bottom -= op.reslut_rect.top;
		}
		break;

	case WM_LBUTTONDOWN:
		if (op.reslut_full_screen != 0) {
			break;
		}
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.reslut_size_width && HIWORD(lParam) < op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPLEFT, lParam);
		} else if (LOWORD(lParam) < op.reslut_size_width && HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMLEFT, lParam);

		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width && HIWORD(lParam) < op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPRIGHT, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width && HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMRIGHT, lParam);

		} else if (LOWORD(lParam) < op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTLEFT, lParam);
		} else if (HIWORD(lParam) < op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOP, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTRIGHT, lParam);
		} else if (HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOM, lParam);
		} else {
			/* EBhËړJn */
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
		}

	case WM_LBUTTONUP:
	case WM_MOUSEMOVE:
		if (op.reslut_full_screen != 0) {
			break;
		}
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.reslut_size_width && HIWORD(lParam) < op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
		} else if (LOWORD(lParam) < op.reslut_size_width && HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));

		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width && HIWORD(lParam) < op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));
		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width && HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));

		} else if (LOWORD(lParam) < op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) < op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else if (LOWORD(lParam) >= rect.right - op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) >= rect.bottom - op.reslut_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else {
			SetCursor(LoadCursor(NULL, IDC_ARROW));
		}
		break;

	case WM_RBUTTONUP:
		SendMessage(hWnd, WM_CONTEXTMENU, (WPARAM)hWnd, lParam);
		break;

	case WM_PAINT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf->draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_ERASEBKGND:
		return TRUE;

	case WM_RESULT_DRAW_INIT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_RESULT_REDRAW:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_result(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_CONTEXTMENU:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			HMENU hMenu;
			POINT apos;
			DWORD st, en;
			WORD lang;
			int i;

			SendMessage(hWnd, EM_GETSEL, (WPARAM)&st, (LPARAM)&en);
			lang = PRIMARYLANGID(LANGIDFROMLCID(GetThreadLocale()));

			// j[̍쐬
			hMenu = CreatePopupMenu();
			if (op.reslut_full_screen != 0) {
				AppendMenu(hMenu, MF_STRING, 3,
					(lang != LANG_JAPANESE) ? TEXT("End &Fullscreen") : TEXT("tXN[(&F)"));
			} else {
				AppendMenu(hMenu, MF_STRING, 3,
					(lang != LANG_JAPANESE) ? TEXT("&Fullscreen") : TEXT("tXN[(&F)"));
			}
			AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
			AppendMenu(hMenu, MF_STRING, 1,
				(lang != LANG_JAPANESE) ? TEXT("&Close") : TEXT("(&C)"));

			// j[̕\
			GetCursorPos((LPPOINT)&apos);
			i = TrackPopupMenu(hMenu, TPM_TOPALIGN | TPM_RETURNCMD, apos.x, apos.y, 0, hWnd, NULL);
			DestroyMenu(hMenu);
			switch (i) {
			case 0:
				break;
			case 1:
				SendMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case 3:
				if (op.reslut_full_screen != 0) {
					op.reslut_full_screen = 0;
					ShowWindow(hWnd, SW_SHOWNORMAL);
				} else {
					op.reslut_full_screen = 1;
					ShowWindow(hWnd, SW_MAXIMIZE);
				}
				break;
			}
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * result_window_regist - EBhENX̓o^
 */
BOOL result_window_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)result_window_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = RESLUT_WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * result_window_create - EBhE̍쐬
 */
HWND result_window_create(const HINSTANCE hInstance, SCORE_INFO *si)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindowEx(0,
		RESLUT_WINDOW_CLASS,
		RESLUT_WINDOW_TITLE,
		WS_POPUP,// | WS_THICKFRAME,
		op.reslut_rect.left,
		op.reslut_rect.top,
		op.reslut_rect.right,
		op.reslut_rect.bottom,
		NULL, NULL, hInstance, si);
	if (hWnd == NULL) {
		return NULL;
	}
	if (op.reslut_full_screen != 0) {
		ShowWindow(hWnd, SW_MAXIMIZE);
	} else {
		ShowWindow(hWnd, SW_SHOW);
	}
	return hWnd;
}
/* End of source */
