/*
 * n01_score
 *
 * window.c
 *
 * Copyright (C) 1996-2007 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <tchar.h>

#include "Memory.h"
#include "String.h"
#include "font.h"
#include "window.h"
#include "score_window.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("n01_score_wnd")
#define WINDOW_TITLE				TEXT("n01 Score")

#define DRAW_LEFT					0
#define DRAW_RIGHT					1
#define DRAW_CENTER					2

#define ID_DRAW_TIMER				1
#define TIMER_INTERVAL				1

/* Global Variables */
extern HINSTANCE hInst;

OPTION op;

typedef struct _DRAW_BUFFER {
	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;

	HDC ellipse_dc;
	HBITMAP ellipse_bmp;
	HBITMAP ellipse_ret_bmp;

	HDC current_dc;
	HBITMAP current_bmp;
	HBITMAP current_ret_bmp;

	HFONT head_font;
	HFONT title_font;
	HFONT name_font;
	HFONT score_font;
	int font_size;

	HBRUSH back_brush;
	HBRUSH header_back_brush;
	HBRUSH name_back_brush;
	HBRUSH sets_back_brush;
	HBRUSH legs_back_brush;
	HBRUSH score_back_brush;
	HBRUSH current_mark_brush;
	HBRUSH current_back_brush;

	HBRUSH first_brush;
	HPEN first_pen;

	BOOL current;

	SCORE_INFO *si;
} DRAW_BUFFER;

/* Local Function Prototypes */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_score(const HWND hWnd, const DRAW_BUFFER *bf);
static LRESULT CALLBACK score_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * draw_init - `̏
 */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf)
{
	HDC hdc;
	RECT rect;

	GetClientRect(hWnd, &rect);

	hdc = GetDC(hWnd);
	bf->draw_bmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
	bf->draw_ret_bmp = SelectObject(bf->draw_dc, bf->draw_bmp);
	bf->ellipse_bmp = CreateCompatibleBitmap(hdc, (rect.bottom / 10) * 2, (rect.bottom / 10) * 2);
	bf->ellipse_ret_bmp = SelectObject(bf->ellipse_dc, bf->ellipse_bmp);
	bf->current_bmp = CreateCompatibleBitmap(hdc, (rect.bottom / 5) * 2, (rect.bottom / 5) * 2);
	bf->current_ret_bmp = SelectObject(bf->current_dc, bf->current_bmp);
	ReleaseDC(hWnd, hdc);

	bf->font_size = rect.bottom;
	bf->head_font = font_create(op.score_font_name, bf->font_size / 7, FW_BOLD, FALSE, FALSE);
	bf->title_font = font_create_menu(bf->font_size / 7, FW_BOLD, FALSE);
	bf->name_font = font_create_menu(bf->font_size / 5, FW_BOLD, FALSE);
	bf->score_font = font_create(op.score_font_name, bf->font_size / 5, FW_BOLD, FALSE, FALSE);
	return TRUE;
}

/*
 * draw_free - `̉
 */
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf)
{
	if (bf->draw_dc != NULL) {
		SelectObject(bf->draw_dc, bf->draw_ret_bmp);
		DeleteObject(bf->draw_bmp);
		bf->draw_bmp = NULL;
	}
	if (bf->ellipse_dc != NULL) {
		SelectObject(bf->ellipse_dc, bf->ellipse_ret_bmp);
		DeleteObject(bf->ellipse_bmp);
		bf->ellipse_bmp = NULL;
	}
	if (bf->current_dc != NULL) {
		SelectObject(bf->current_dc, bf->current_ret_bmp);
		DeleteObject(bf->current_bmp);
		bf->current_bmp = NULL;
	}
	if (bf->score_font != NULL) {
		DeleteObject(bf->score_font);
		bf->score_font = NULL;
	}
	if (bf->name_font != NULL) {
		DeleteObject(bf->name_font);
		bf->name_font = NULL;
	}
	if (bf->title_font != NULL) {
		DeleteObject(bf->title_font);
		bf->title_font = NULL;
	}
	if (bf->head_font != NULL) {
		DeleteObject(bf->head_font);
		bf->head_font = NULL;
	}
	return TRUE;
}

/*
 * draw_text - eLXg̕`
 */
static void draw_text(const HDC hdc, const TCHAR *str, const int len, const RECT *rect, const int type)
{
	SIZE sz;
	int left, top;

	GetTextExtentPoint32(hdc, str, len, &sz);
	top = rect->top + (rect->bottom - rect->top - sz.cy) / 2;
	switch (type) {
	case DRAW_CENTER:
		left = rect->left + (rect->right - rect->left - sz.cx) / 2;
		break;

	case DRAW_RIGHT:
		left = rect->right - sz.cx;
		break;

	case DRAW_LEFT:
	default:
		left = rect->left;
		break;
	}
	ExtTextOut(hdc, left, top, ETO_CLIPPED, rect, str, len, NULL);
}

/*
 * draw_score - cXRA̕`
 */
static BOOL draw_score(const HWND hWnd, const DRAW_BUFFER *bf)
{
	RECT rect;
	RECT head_rect;
	RECT name_rect[2], sets_rect[2], legs_rect[2], score_rect[2], current_rect[2];
	RECT tmp_rect;
	HFONT ret_font;
	HPEN ret_pen;
	HBRUSH ret_brush;
	TCHAR buf[BUF_SIZE];
	int i;
	int top;
	int height;
	int width;

	GetClientRect(hWnd, &rect);
	FillRect(bf->draw_dc, &rect, bf->back_brush);

	rect.left += op.score_line_width;
	rect.top += op.score_line_width;
	rect.right -= op.score_line_width;
	rect.bottom -= op.score_line_width;

	// Header
	SetRect(&head_rect, rect.left, rect.top, rect.right, rect.bottom / 4);
	FillRect(bf->draw_dc, &head_rect, bf->header_back_brush);

	top = head_rect.bottom + op.score_line_width;
	height = top + (rect.bottom - top) / 2;
	for (i = 0; i < 2; i++) {
		// `
		SetRect(&current_rect[i], rect.right - rect.bottom * 10 / 30, top, rect.right, height);
		SetRect(&score_rect[i], current_rect[i].left - rect.bottom * 10 / 15, top, current_rect[i].left - op.score_line_width, height);
		SetRect(&legs_rect[i], score_rect[i].left - rect.bottom * 10 / 18, top, score_rect[i].left - op.score_line_width, height);
		if (op.score_visible_set != 0) {
			SetRect(&sets_rect[i], legs_rect[i].left - rect.bottom * 10 / 18, top, legs_rect[i].left - op.score_line_width, height);
			SetRect(&name_rect[i], rect.left, top, sets_rect[i].left - op.score_line_width, height);
		} else {
			SetRect(&name_rect[i], rect.left, top, legs_rect[i].left - op.score_line_width, height);
		}

		// hԂ
		FillRect(bf->draw_dc, &name_rect[i], bf->name_back_brush);
		if (op.score_visible_set != 0) {
			FillRect(bf->draw_dc, &sets_rect[i], bf->sets_back_brush);
		}
		FillRect(bf->draw_dc, &legs_rect[i], bf->legs_back_brush);
		FillRect(bf->draw_dc, &score_rect[i], bf->score_back_brush);
		FillRect(bf->draw_dc, &current_rect[i], bf->current_back_brush);

		top = height + op.score_line_width;
		height = rect.bottom;
	}
	
	SetTextColor(bf->draw_dc, op.score_head_text);

	ret_font = SelectObject(bf->draw_dc, bf->title_font);
	SetRect(&tmp_rect, name_rect[0].left + rect.bottom / 30, head_rect.top, name_rect[0].right, head_rect.bottom);
	DrawText(bf->draw_dc, op.score_title, lstrlen(op.score_title), &tmp_rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
	SelectObject(bf->draw_dc, ret_font);

	ret_font = SelectObject(bf->draw_dc, bf->head_font);
	if (op.score_visible_set != 0) {
		SetRect(&tmp_rect, sets_rect[0].left, head_rect.top, sets_rect[0].right, head_rect.bottom);
		draw_text(bf->draw_dc, TEXT("Sets"), lstrlen(TEXT("Sets")), &tmp_rect, DRAW_CENTER);
	}
	SetRect(&tmp_rect, legs_rect[0].left, head_rect.top, legs_rect[0].right, head_rect.bottom);
	draw_text(bf->draw_dc, TEXT("Legs"), lstrlen(TEXT("Legs")), &tmp_rect, DRAW_CENTER);
	SelectObject(bf->draw_dc, ret_font);

	// `
	for (i = 0; i < 2; i++) {
		RECT name_draw_rect;
		// Name
		ret_font = SelectObject(bf->draw_dc, bf->name_font);
		SetRect(&name_draw_rect, name_rect[i].left + rect.bottom / 30, name_rect[i].top, name_rect[i].right - rect.bottom / 10 - rect.bottom / 10 / 2, name_rect[i].bottom);
		SetTextColor(bf->draw_dc, op.score_name_text);
		DrawText(bf->draw_dc, bf->si->player[i].name, lstrlen(bf->si->player[i].name), &name_draw_rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		SelectObject(bf->draw_dc, ret_font);

		if (bf->si->leg[bf->si->current_leg].first == i) {
			// First
			RECT draw_rect;

			width = rect.bottom / 10;
			SetRect(&draw_rect, 0, 0, width * 2, width * 2);
			FillRect(bf->ellipse_dc, &draw_rect, bf->name_back_brush);
			ret_brush = SelectObject(bf->ellipse_dc, bf->first_brush);
			ret_pen = SelectObject(bf->ellipse_dc, bf->first_pen);
			Ellipse(bf->ellipse_dc, 0, 0, width * 2, width * 2);
			SelectObject(bf->ellipse_dc, ret_brush);
			SelectObject(bf->ellipse_dc, ret_pen);

			StretchBlt(bf->draw_dc,
				name_rect[i].right - width - width / 2,
				name_rect[i].top + (name_rect[i].bottom - name_rect[i].top - width) / 2,
				width, width,
				bf->ellipse_dc, 0, 0, width * 2, width * 2, SRCCOPY);
		}

		ret_font = SelectObject(bf->draw_dc, bf->score_font);
		if (op.score_visible_set != 0) {
			// Set
			SetTextColor(bf->draw_dc, op.score_sets_text);
			_itot(bf->si->player[i].sets, buf, 10);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &sets_rect[i], DRAW_CENTER);
		}

		// Leg
		SetTextColor(bf->draw_dc, op.score_legs_text);
		_itot(bf->si->player[i].legs, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &legs_rect[i], DRAW_CENTER);

		// Score
		SetTextColor(bf->draw_dc, op.score_score_text);
		_itot(bf->si->player[i].left, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &score_rect[i], DRAW_CENTER);
		SelectObject(bf->draw_dc, ret_font);

		if (bf->si->leg[bf->si->current_leg].current_player == i) {
			// Current
			RECT draw_rect;
			int w, h;

			w = rect.bottom / 5;
			h = rect.bottom / 5;
			SetRect(&draw_rect, 0, 0, w * 2, h * 2);
			FillRect(bf->current_dc, &draw_rect, bf->current_back_brush);

			BeginPath(bf->current_dc);
			MoveToEx(bf->current_dc, draw_rect.right, draw_rect.top, (LPPOINT) NULL);
			LineTo(bf->current_dc, draw_rect.right, draw_rect.bottom);
			LineTo(bf->current_dc, draw_rect.left, draw_rect.top + (draw_rect.bottom - draw_rect.top) / 2);
			LineTo(bf->current_dc, draw_rect.right, draw_rect.top);
			EndPath(bf->current_dc);

			ret_brush = SelectObject(bf->current_dc, bf->current_mark_brush);
			FillPath(bf->current_dc);
			SelectObject(bf->current_dc, ret_brush);

			StretchBlt(bf->draw_dc,
				current_rect[i].left + (current_rect[i].right - current_rect[i].left) / 2 - w / 2,
				current_rect[i].top + (current_rect[i].bottom - current_rect[i].top) / 2 - h / 2,
				w, h, bf->current_dc, 0, 0, w * 2, h * 2, SRCCOPY);
		}
	}
	return TRUE;
}

/*
 * set_title_proc - EBhEvV[W
 */
static BOOL CALLBACK set_title_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
	case WM_INITDIALOG:
		SendDlgItemMessage(hDlg, IDC_EDIT_TITLE, WM_SETTEXT, 0, (LPARAM)op.score_title);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, FALSE);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			SendDlgItemMessage(hDlg, IDC_EDIT_TITLE, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)op.score_title);
			EndDialog(hDlg, TRUE);
			break;

		case IDCANCEL:
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * score_window_proc - XRA\EBhEvV[W
 */
static LRESULT CALLBACK score_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	DRAW_BUFFER *bf;
	SCORE_INFO *si;
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;

	switch (msg) {
	case WM_CREATE:
		si = (SCORE_INFO *)((CREATESTRUCT *)lParam)->lpCreateParams;
		if (si == NULL) {
			return -1;
		}
		bf = (DRAW_BUFFER *)mem_calloc(sizeof(DRAW_BUFFER));
		if (bf == NULL) {
			return -1;
		}
		bf->si = si;

		// `p
		hdc = GetDC(hWnd);
		bf->draw_dc = CreateCompatibleDC(hdc);
		bf->ellipse_dc = CreateCompatibleDC(hdc);
		bf->current_dc = CreateCompatibleDC(hdc);
		ReleaseDC(hWnd, hdc);
		draw_init(hWnd, bf);
		SetBkMode(bf->draw_dc, TRANSPARENT);
		bf->back_brush = CreateSolidBrush(op.score_line_color);
		bf->header_back_brush = CreateSolidBrush(op.score_head_back);
		bf->name_back_brush = CreateSolidBrush(op.score_name_back);
		bf->sets_back_brush = CreateSolidBrush(op.score_sets_back);
		bf->legs_back_brush = CreateSolidBrush(op.score_legs_back);
		bf->score_back_brush = CreateSolidBrush(op.score_score_back);
		bf->current_mark_brush = CreateSolidBrush(op.score_current_mark);
		bf->current_back_brush = CreateSolidBrush(op.score_current_back);
		bf->first_brush = CreateSolidBrush(op.score_first_mark);
		bf->first_pen = CreatePen(PS_SOLID, 1, op.score_first_mark);
		if (GetDeviceCaps(bf->draw_dc, BITSPIXEL) >= 16) {
			SetStretchBltMode(bf->draw_dc, HALFTONE);
			SetBrushOrgEx(bf->draw_dc, 0, 0, NULL);
		}
		draw_score(hWnd, bf);

		SetWindowLong(hWnd, GWL_USERDATA, (LONG)bf);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			draw_free(hWnd, bf);
			if (bf->draw_dc != NULL) {
				DeleteDC(bf->draw_dc);
				bf->draw_dc = NULL;
			}
			if (bf->ellipse_dc != NULL) {
				DeleteDC(bf->ellipse_dc);
				bf->ellipse_dc = NULL;
			}
			if (bf->current_dc != NULL) {
				DeleteDC(bf->current_dc);
				bf->current_dc = NULL;
			}
			DeleteObject(bf->back_brush);
			DeleteObject(bf->header_back_brush);
			DeleteObject(bf->name_back_brush);
			DeleteObject(bf->sets_back_brush);
			DeleteObject(bf->legs_back_brush);
			DeleteObject(bf->score_back_brush);
			DeleteObject(bf->current_mark_brush);
			DeleteObject(bf->current_back_brush);
			DeleteObject(bf->first_brush);
			DeleteObject(bf->first_pen);
			mem_free(&bf);
		}
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SIZE:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		draw_score(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_EXITSIZEMOVE:
		// TCYύX
		if (IsWindowVisible(hWnd) != 0 && IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
			GetWindowRect(hWnd, (LPRECT)&op.score_window_rect);
			op.score_window_rect.right -= op.score_window_rect.left;
			op.score_window_rect.bottom -= op.score_window_rect.top;
		}
		break;

	case WM_LBUTTONDOWN:
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.score_size_width && HIWORD(lParam) < op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPLEFT, lParam);
		} else if (LOWORD(lParam) < op.score_size_width && HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMLEFT, lParam);

		} else if (LOWORD(lParam) >= rect.right - op.score_size_width && HIWORD(lParam) < op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPRIGHT, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.score_size_width && HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMRIGHT, lParam);

		} else if (LOWORD(lParam) < op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTLEFT, lParam);
		} else if (HIWORD(lParam) < op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOP, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTRIGHT, lParam);
		} else if (HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOM, lParam);
		} else {
			/* EBhËړJn */
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
		}

	case WM_LBUTTONUP:
	case WM_MOUSEMOVE:
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.score_size_width && HIWORD(lParam) < op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
		} else if (LOWORD(lParam) < op.score_size_width && HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));

		} else if (LOWORD(lParam) >= rect.right - op.score_size_width && HIWORD(lParam) < op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));
		} else if (LOWORD(lParam) >= rect.right - op.score_size_width && HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));

		} else if (LOWORD(lParam) < op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) < op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else if (LOWORD(lParam) >= rect.right - op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) >= rect.bottom - op.score_size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else {
			SetCursor(LoadCursor(NULL, IDC_ARROW));
		}
		break;

	case WM_RBUTTONUP:
		SendMessage(hWnd, WM_CONTEXTMENU, (WPARAM)hWnd, lParam);
		break;

	case WM_PAINT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf->draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_ERASEBKGND:
		return TRUE;

	case WM_SCORE_DRAW_INIT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_SCORE_REDRAW:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_score(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SCORE_REDRAW_DELAY:
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		SetTimer(hWnd, ID_DRAW_TIMER, TIMER_INTERVAL, NULL);
		break;

	case WM_TIMER:
		KillTimer(hWnd, wParam);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_CONTEXTMENU:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			HMENU hMenu;
			POINT apos;
			DWORD st, en;
			WORD lang;
			int i;

			SendMessage(hWnd, EM_GETSEL, (WPARAM)&st, (LPARAM)&en);
			lang = PRIMARYLANGID(LANGIDFROMLCID(GetThreadLocale()));

			// j[̍쐬
			hMenu = CreatePopupMenu();
			AppendMenu(hMenu, MF_STRING, 3,
				(lang != LANG_JAPANESE) ? TEXT("Edit &Title...") : TEXT("^Cg̐ݒ(&T)..."));
			if (op.score_visible_set != 0) {
				AppendMenu(hMenu, MF_STRING, 1,
					(lang != LANG_JAPANESE) ? TEXT("Hide &Sets") : TEXT("ZbgB(&S)"));
			} else {
				AppendMenu(hMenu, MF_STRING, 1,
					(lang != LANG_JAPANESE) ? TEXT("Show &Sets") : TEXT("Zbg\(&S)"));
			}
			AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
			AppendMenu(hMenu, MF_STRING, 2,
				(lang != LANG_JAPANESE) ? TEXT("&Close") : TEXT("(&C)"));

			// j[̕\
			GetCursorPos((LPPOINT)&apos);
			i = TrackPopupMenu(hMenu, TPM_TOPALIGN | TPM_RETURNCMD, apos.x, apos.y, 0, hWnd, NULL);
			DestroyMenu(hMenu);
			switch (i) {
			case 0:
				break;
			case 1:
				op.score_visible_set = (op.score_visible_set == 0) ? 1 : 0;
				draw_free(hWnd, bf);
				draw_init(hWnd, bf);
				draw_score(hWnd, bf);
				InvalidateRect(hWnd, NULL, FALSE);
				UpdateWindow(hWnd);
				break;
			case 2:
				SendMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case 3:
				if (DialogBoxParam((HINSTANCE)GetClassLong(hWnd, GCL_HMODULE), MAKEINTRESOURCE(IDD_DIALOG_TITLE), hWnd, set_title_proc, 0) == FALSE) {
					break;
				}
				SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
				break;
			}
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_window_regist - EBhENX̓o^
 */
BOOL score_window_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)score_window_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_window_create - EBhE̍쐬
 */
HWND score_window_create(const HINSTANCE hInstance, SCORE_INFO *si)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindowEx(0,
		WINDOW_CLASS,
		WINDOW_TITLE,
		WS_POPUP,// | WS_THICKFRAME,
		op.score_window_rect.left,
		op.score_window_rect.top,
		op.score_window_rect.right,
		op.score_window_rect.bottom,
		NULL, NULL, hInstance, si);
	if (hWnd == NULL) {
		return NULL;
	}
	ShowWindow(hWnd, SW_SHOW);
	return hWnd;
}
/* End of source */
