/*
 * n01_view
 *
 * window.c
 *
 * Copyright (C) 1996-2016 by Ohno Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <tchar.h>

#include "Memory.h"
#include "String.h"
#include "font.h"
#include "window.h"

//#include "resource.h"

/* Define */
#define WINDOW_CLASS				TEXT("n01_view_wnd")
#define WINDOW_TITLE				TEXT("n01 View")

#define DRAW_LEFT					0
#define DRAW_RIGHT					1
#define DRAW_CENTER					2

#define NAME_HEIGHT					15
#define SCORE_WIDTH					30
#define LEG_WIDTH					30
#define LEG_HEIGHT					15

#define PAD_WIDTH					6
#define PAD_HEIGHT					6

#define ID_DRAW_TIMER				1
#define TIMER_INTERVAL				1

/* Global Variables */
extern HINSTANCE hInst;

OPTION op;

typedef struct _DRAW_BUFFER {
	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;

	HBITMAP back_bmp;
	HBITMAP num_l_bmp;
	HBITMAP num_s_bmp;

	HDC score_dc;
	HBITMAP socre_bmp;
	HBITMAP score_ret_bmp;

	HDC back_dc;
	HBITMAP back_ret_bmp;
	BITMAP back_bitmap;

	HDC num_l_dc;
	HBITMAP num_l_ret_bmp;
	BITMAP num_l_bitmap;
	int num_size_l;

	HDC num_s_dc;
	HBITMAP num_s_ret_bmp;
	BITMAP num_s_bitmap;
	int num_size_s;

	HFONT head_font;
	HFONT name_font;
	HFONT score_font;
	HFONT leg_font;

	HBRUSH back_brush;
	HBRUSH name_back_brush;
	HBRUSH sets_back_brush;
	HBRUSH legs_back_brush;
	HBRUSH score_back_brush;

	SCORE_INFO *si;
} DRAW_BUFFER;

/* Local Function Prototypes */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_score(const HWND hWnd, const DRAW_BUFFER *bf);
static LRESULT CALLBACK score_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * draw_init - `̏
 */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf)
{
	HDC hdc;
	RECT rect;

	GetClientRect(hWnd, &rect);

	hdc = GetDC(hWnd);
	bf->draw_bmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
	bf->draw_ret_bmp = SelectObject(bf->draw_dc, bf->draw_bmp);
	ReleaseDC(hWnd, hdc);

	GetObject(bf->back_bmp, sizeof(BITMAP), &bf->back_bitmap);

	bf->score_dc = CreateCompatibleDC(bf->draw_dc);
	bf->socre_bmp = CreateCompatibleBitmap(bf->draw_dc, bf->back_bitmap.bmWidth, bf->back_bitmap.bmHeight);
	bf->score_ret_bmp = SelectObject(bf->score_dc, bf->socre_bmp);
	SetBkMode(bf->score_dc, TRANSPARENT);

	bf->back_dc = CreateCompatibleDC(bf->draw_dc);
	bf->back_ret_bmp = SelectObject(bf->back_dc, bf->back_bmp);

	bf->num_l_dc = CreateCompatibleDC(bf->draw_dc);
	bf->num_l_ret_bmp = SelectObject(bf->num_l_dc, bf->num_l_bmp);
	GetObject(bf->num_l_bmp, sizeof(BITMAP), &bf->num_l_bitmap);
	bf->num_size_l = bf->num_l_bitmap.bmWidth / 10;

	bf->num_s_dc = CreateCompatibleDC(bf->draw_dc);
	bf->num_s_ret_bmp = SelectObject(bf->num_s_dc, bf->num_s_bmp);
	GetObject(bf->num_s_bmp, sizeof(BITMAP), &bf->num_s_bitmap);
	bf->num_size_s = bf->num_s_bitmap.bmWidth / 10;

	if (op.name_font_size > 0) {
		bf->name_font = font_create_menu(op.name_font_size, FW_BOLD, FALSE);
	} else {
		bf->name_font = font_create_menu(rect.right / 22, FW_BOLD, FALSE);
	}
	bf->head_font = font_create(op.font_name, rect.right / 22, FW_BOLD, FALSE, FALSE);
	bf->score_font = font_create(op.font_name, rect.right / 11, FW_BOLD, FALSE, FALSE);
	bf->leg_font = font_create(op.font_name, rect.right / 17, FW_BOLD, FALSE, FALSE);
	return TRUE;
}

/*
 * draw_free - `̉
 */
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf)
{
	if (bf->num_s_dc != NULL) {
		SelectObject(bf->num_s_dc, bf->num_s_ret_bmp);
		DeleteDC(bf->num_s_dc);
	}
	if (bf->num_l_dc != NULL) {
		SelectObject(bf->num_l_dc, bf->num_l_ret_bmp);
		DeleteDC(bf->num_l_dc);
	}
	if (bf->back_dc != NULL) {
		SelectObject(bf->back_dc, bf->back_ret_bmp);
		DeleteDC(bf->back_dc);
	}
	if (bf->score_dc != NULL) {
		SelectObject(bf->score_dc, bf->score_ret_bmp);
		DeleteObject(bf->socre_bmp);
		DeleteDC(bf->score_dc);
	}
	
	if (bf->draw_dc != NULL) {
		SelectObject(bf->draw_dc, bf->draw_ret_bmp);
		DeleteObject(bf->draw_bmp);
		bf->draw_bmp = NULL;
	}
	if (bf->score_font != NULL) {
		DeleteObject(bf->score_font);
		bf->score_font = NULL;
	}
	if (bf->head_font != NULL) {
		DeleteObject(bf->head_font);
		bf->head_font = NULL;
	}
	if (bf->name_font != NULL) {
		DeleteObject(bf->name_font);
		bf->name_font = NULL;
	}
	if (bf->leg_font != NULL) {
		DeleteObject(bf->leg_font);
		bf->leg_font = NULL;
	}
	return TRUE;
}

/*
 * draw_text - eLXg̕`
 */
static void draw_text(const HDC hdc, const TCHAR *str, const int len, const RECT *rect, const int type)
{
	SIZE sz;
	int left, top;

	GetTextExtentPoint32(hdc, str, len, &sz);
	top = rect->top + (rect->bottom - rect->top - sz.cy) / 2;
	switch (type) {
	case DRAW_CENTER:
		left = rect->left + (rect->right - rect->left - sz.cx) / 2;
		break;

	case DRAW_RIGHT:
		left = rect->right - sz.cx - 2;
		break;

	case DRAW_LEFT:
	default:
		left = rect->left + 2;
		break;
	}
	ExtTextOut(hdc, left, top, ETO_CLIPPED, rect, str, len, NULL);
}

/*
 * draw_score_bmp - cXRA̕` (rbg}bv)
 */
static BOOL draw_score_bmp(const HWND hWnd, const DRAW_BUFFER *bf)
{
	RECT rect;
	RECT name_rect;
	HFONT ret_font;
	TCHAR buf[BUF_SIZE];
	int i, j;

	GetClientRect(hWnd, &rect);
	BitBlt(bf->score_dc, 0, 0, bf->back_bitmap.bmWidth, bf->back_bitmap.bmHeight, bf->back_dc, 0, 0, SRCCOPY);

	for (i = 0; i < 2; i++) {
		// Name
		SetRect(&name_rect, op.name_left[i], op.name_top[i], op.name_right[i], op.name_bottom[i]);
		ret_font = SelectObject(bf->score_dc, bf->name_font);
		SetTextColor(bf->score_dc, op.name_text);
		DrawText(bf->score_dc, bf->si->player[i].name, lstrlen(bf->si->player[i].name), &name_rect, DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		SelectObject(bf->score_dc, ret_font);

		// Score
		if (op.score_x[i] > 0 && op.score_y[i] > 0) {
			_itot(bf->si->player[i].left, buf, 10);
			for (j = 0; j < lstrlen(buf); j++) {
				int left = op.score_x[i] + (bf->num_size_l * 3) / 2 - (bf->num_size_l * lstrlen(buf)) / 2 + bf->num_size_l * j;
				BitBlt(bf->score_dc, left, op.score_y[i], bf->num_size_l, bf->num_l_bitmap.bmHeight,
					bf->num_l_dc, (buf[j] - TEXT('0')) * bf->num_size_l, 0, SRCCOPY);
			}
		}
		// Sets
		if (op.sets_x[i] > 0 && op.sets_y[i] > 0) {
			_itot(bf->si->player[i].sets, buf, 10);
			for (j = 0; j < lstrlen(buf); j++) {
				int left = op.sets_x[i] + (bf->num_size_s * 3) / 2 - (bf->num_size_s * lstrlen(buf)) / 2 + bf->num_size_s * j;
				BitBlt(bf->score_dc, left, op.sets_y[i], bf->num_size_s, bf->num_s_bitmap.bmHeight,
					bf->num_s_dc, (buf[j] - TEXT('0')) * bf->num_size_s, 0, SRCCOPY);
			}
		}
		// Legs
		if (op.legs_x[i] > 0 && op.legs_y[i] > 0) {
			_itot(bf->si->player[i].legs, buf, 10);
			for (j = 0; j < lstrlen(buf); j++) {
				int left = op.legs_x[i] + (bf->num_size_s * 3) / 2 - (bf->num_size_s * lstrlen(buf)) / 2 + bf->num_size_s * j;
				BitBlt(bf->score_dc, left, op.legs_y[i], bf->num_size_s, bf->num_s_bitmap.bmHeight,
					bf->num_s_dc, (buf[j] - TEXT('0')) * bf->num_size_s, 0, SRCCOPY);
			}
		}
	}
	SetStretchBltMode(bf->draw_dc, HALFTONE);
	SetBrushOrgEx(bf->draw_dc, 0, 0, NULL);
	StretchBlt(bf->draw_dc, 0, 0, rect.right, rect.bottom, bf->score_dc, 0, 0, bf->back_bitmap.bmWidth, bf->back_bitmap.bmHeight, SRCCOPY);
	return TRUE;
}

/*
 * draw_score - cXRA̕`
 */
static BOOL draw_score(const HWND hWnd, const DRAW_BUFFER *bf)
{
	RECT rect;
	RECT name_rect[2], sets_rect[2], legs_rect[2], score_rect[2];
	RECT tmp_rect;
	HFONT ret_font;
	TCHAR buf[BUF_SIZE];
	int i;
	int height;
	int top, bottom;
	int left, right;
	int pad_height;

	if (bf->back_bmp != NULL) {
		return draw_score_bmp(hWnd, bf);
	}

	GetClientRect(hWnd, &rect);
	FillRect(bf->draw_dc, &rect, bf->back_brush);

	if (op.visible_set != 0) {
		pad_height = PAD_HEIGHT;
	} else {
		pad_height = PAD_HEIGHT * 2;
	}

	// name
	top = rect.top + rect.bottom * pad_height / 100;
	bottom = top + rect.bottom * NAME_HEIGHT / 100;
	left = rect.left + rect.right * PAD_WIDTH / 100;
	right = left + rect.right * ((105 / 2) - PAD_WIDTH * 2) / 100;
	SetRect(&name_rect[0], left, top, right, bottom);
	left = rect.left + rect.right * (100 - ((105 / 2) - PAD_WIDTH * 2) - PAD_WIDTH) / 100;
	right = left + rect.right * ((105 / 2) - PAD_WIDTH * 2) / 100;
	SetRect(&name_rect[1], left, top, right, bottom);

	// set
	top = rect.top + rect.bottom * (100 - LEG_HEIGHT - pad_height) / 100;
	bottom = top + rect.bottom * LEG_HEIGHT / 100;
	left = rect.left + rect.right * PAD_WIDTH / 100;
	right = left + rect.right * LEG_WIDTH / 100;
	SetRect(&sets_rect[0], left, top, right, bottom);
	left = rect.left + rect.right * (100 - LEG_WIDTH - PAD_WIDTH) / 100;
	right = left + rect.right * LEG_WIDTH / 100;
	SetRect(&sets_rect[1], left, top, right, bottom);

	// leg
	if (op.visible_set != 0) {
		top = rect.top + rect.bottom * (100 - LEG_HEIGHT - pad_height - LEG_HEIGHT - pad_height) / 100;
		bottom = top + rect.bottom * LEG_HEIGHT / 100;
		left = rect.left + rect.right * PAD_WIDTH / 100;
		right = left + rect.right * LEG_WIDTH / 100;
		SetRect(&legs_rect[0], left, top, right, bottom);
		left = rect.left + rect.right * (100 - LEG_WIDTH - PAD_WIDTH) / 100;
		right = left + rect.right * LEG_WIDTH / 100;
		SetRect(&legs_rect[1], left, top, right, bottom);

		height = 100 - pad_height - NAME_HEIGHT - pad_height - LEG_HEIGHT - pad_height - LEG_HEIGHT - pad_height - pad_height;
	} else {
		CopyMemory(legs_rect, sets_rect, sizeof(RECT) * 2);
		height = 100 - pad_height - NAME_HEIGHT - pad_height - LEG_HEIGHT - pad_height - pad_height;
	}

	// score
	top = rect.top + rect.bottom * (LEG_HEIGHT + pad_height + pad_height) / 100;
	bottom = top + rect.bottom * height / 100;
	left = rect.left + rect.right * PAD_WIDTH / 100;
	right = left + rect.right * SCORE_WIDTH / 100;
	SetRect(&score_rect[0], left, top, right, bottom);
	left = rect.left + rect.right * (100 - SCORE_WIDTH - PAD_WIDTH) / 100;
	right = left + rect.right * SCORE_WIDTH / 100;
	SetRect(&score_rect[1], left, top, right, bottom);

	for (i = 0; i < 2; i++) {
		// hԂ
		FillRect(bf->draw_dc, &name_rect[i], bf->name_back_brush);
		if (op.visible_set != 0) {
			FillRect(bf->draw_dc, &sets_rect[i], bf->sets_back_brush);
		}
		FillRect(bf->draw_dc, &legs_rect[i], bf->legs_back_brush);
		FillRect(bf->draw_dc, &score_rect[i], bf->score_back_brush);
	}
	
	SetTextColor(bf->draw_dc, op.head_text);
	ret_font = SelectObject(bf->draw_dc, bf->head_font);
	SetRect(&tmp_rect, score_rect[0].right, score_rect[0].top, score_rect[1].left, score_rect[0].bottom);
	draw_text(bf->draw_dc, TEXT("SCORE"), lstrlen(TEXT("SCORE")), &tmp_rect, DRAW_CENTER);
	SetRect(&tmp_rect, legs_rect[0].right, legs_rect[0].top, legs_rect[1].left, legs_rect[0].bottom);
	draw_text(bf->draw_dc, TEXT("LEGS"), lstrlen(TEXT("LEGS")), &tmp_rect, DRAW_CENTER);
	if (op.visible_set != 0) {
		SetRect(&tmp_rect, sets_rect[0].right, sets_rect[0].top, sets_rect[1].left, sets_rect[0].bottom);
		draw_text(bf->draw_dc, TEXT("SETS"), lstrlen(TEXT("SETS")), &tmp_rect, DRAW_CENTER);
	}
	SelectObject(bf->draw_dc, ret_font);

	// `
	for (i = 0; i < 2; i++) {
		// Name
		ret_font = SelectObject(bf->draw_dc, bf->name_font);
		SetTextColor(bf->draw_dc, op.name_text);
		DrawText(bf->draw_dc, bf->si->player[i].name, lstrlen(bf->si->player[i].name), &name_rect[i], DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		SelectObject(bf->draw_dc, ret_font);

		ret_font = SelectObject(bf->draw_dc, bf->leg_font);
		if (op.visible_set != 0) {
			// Set
			SetTextColor(bf->draw_dc, op.sets_text);
			_itot(bf->si->player[i].sets, buf, 10);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &sets_rect[i], DRAW_CENTER);
		}
		// Leg
		SetTextColor(bf->draw_dc, op.legs_text);
		_itot(bf->si->player[i].legs, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &legs_rect[i], DRAW_CENTER);
		SelectObject(bf->draw_dc, ret_font);

		// Score
		ret_font = SelectObject(bf->draw_dc, bf->score_font);
		SetTextColor(bf->draw_dc, op.score_text);
		_itot(bf->si->player[i].left, buf, 10);
		draw_text(bf->draw_dc, buf, lstrlen(buf), &score_rect[i], DRAW_CENTER);
		SelectObject(bf->draw_dc, ret_font);
	}
	return TRUE;
}

/*
 * score_window_proc - XRA\EBhEvV[W
 */
static LRESULT CALLBACK score_window_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	DRAW_BUFFER *bf;
	SCORE_INFO *si;
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;

	switch (msg) {
	case WM_CREATE:
		si = (SCORE_INFO *)((CREATESTRUCT *)lParam)->lpCreateParams;
		if (si == NULL) {
			return -1;
		}
		bf = (DRAW_BUFFER *)mem_calloc(sizeof(DRAW_BUFFER));
		if (bf == NULL) {
			return -1;
		}
		bf->si = si;

		// 摜ǂݍ
		if(*op.back_bmp != TEXT('\0') && *op.num_l_bmp != TEXT('\0') && *op.num_s_bmp != TEXT('\0')){
			TCHAR path[MAX_PATH];
			TCHAR tmpPath[MAX_PATH];
			TCHAR *p, *r;

			GetCurrentDirectory(MAX_PATH, path);
			// JgfBNgDLL̏ꏊɈꎞύX
			GetModuleFileName((HMODULE)GetClassLong(hWnd, GCL_HMODULE), tmpPath, MAX_PATH - 1);
			for (p = r = tmpPath; *p != TEXT('\0'); p++) {
#ifndef UNICODE
				if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
					p++;
					continue;
				}
#endif	// UNICODE
				if (*p == TEXT('\\') || *p == TEXT('/')) {
					r = p;
				}
			}
			*r = TEXT('\0');
			SetCurrentDirectory(tmpPath);
			bf->back_bmp = LoadImage(NULL, op.back_bmp, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			bf->num_l_bmp = LoadImage(NULL, op.num_l_bmp, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			bf->num_s_bmp = LoadImage(NULL, op.num_s_bmp, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			SetCurrentDirectory(path);
		}

		// `p
		hdc = GetDC(hWnd);
		bf->draw_dc = CreateCompatibleDC(hdc);
		ReleaseDC(hWnd, hdc);
		draw_init(hWnd, bf);
		SetBkMode(bf->draw_dc, TRANSPARENT);
		bf->back_brush = CreateSolidBrush(op.line_color);
		bf->name_back_brush = CreateSolidBrush(op.name_back);
		bf->sets_back_brush = CreateSolidBrush(op.sets_back);
		bf->legs_back_brush = CreateSolidBrush(op.legs_back);
		bf->score_back_brush = CreateSolidBrush(op.score_back);
		draw_score(hWnd, bf);

		SetWindowLong(hWnd, GWL_USERDATA, (LONG)bf);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			draw_free(hWnd, bf);
			if (bf->draw_dc != NULL) {
				DeleteDC(bf->draw_dc);
				bf->draw_dc = NULL;
			}
			if (bf->back_bmp != NULL) {
				DeleteObject(bf->back_bmp);
			}
			if (bf->num_l_bmp != NULL) {
				DeleteObject(bf->num_l_bmp);
			}
			if (bf->num_s_bmp != NULL) {
				DeleteObject(bf->num_s_bmp);
			}
			DeleteObject(bf->back_brush);
			DeleteObject(bf->name_back_brush);
			DeleteObject(bf->sets_back_brush);
			DeleteObject(bf->legs_back_brush);
			DeleteObject(bf->score_back_brush);
			mem_free(&bf);
		}
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SIZE:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		draw_score(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_EXITSIZEMOVE:
		if (op.full_screen != 0) {
			break;
		}
		// TCYύX
		if (IsWindowVisible(hWnd) != 0 && IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
			GetWindowRect(hWnd, (LPRECT)&op.window_rect);
			op.window_rect.right -= op.window_rect.left;
			op.window_rect.bottom -= op.window_rect.top;
		}
		break;

	case WM_LBUTTONDOWN:
		if (op.full_screen != 0) {
			break;
		}
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.size_width && HIWORD(lParam) < op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPLEFT, lParam);
		} else if (LOWORD(lParam) < op.size_width && HIWORD(lParam) >= rect.bottom - op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMLEFT, lParam);

		} else if (LOWORD(lParam) >= rect.right - op.size_width && HIWORD(lParam) < op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOPRIGHT, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.size_width && HIWORD(lParam) >= rect.bottom - op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOMRIGHT, lParam);

		} else if (LOWORD(lParam) < op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTLEFT, lParam);
		} else if (HIWORD(lParam) < op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTTOP, lParam);
		} else if (LOWORD(lParam) >= rect.right - op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTRIGHT, lParam);
		} else if (HIWORD(lParam) >= rect.bottom - op.size_width) {
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTBOTTOM, lParam);
		} else {
			/* EBhËړJn */
			SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
		}

	case WM_LBUTTONUP:
	case WM_MOUSEMOVE:
		if (op.full_screen != 0) {
			break;
		}
		GetClientRect(hWnd, &rect);
		if (LOWORD(lParam) < op.size_width && HIWORD(lParam) < op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
		} else if (LOWORD(lParam) < op.size_width && HIWORD(lParam) >= rect.bottom - op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));

		} else if (LOWORD(lParam) >= rect.right - op.size_width && HIWORD(lParam) < op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENESW));
		} else if (LOWORD(lParam) >= rect.right - op.size_width && HIWORD(lParam) >= rect.bottom - op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE));

		} else if (LOWORD(lParam) < op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) < op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else if (LOWORD(lParam) >= rect.right - op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		} else if (HIWORD(lParam) >= rect.bottom - op.size_width) {
			SetCursor(LoadCursor(NULL, IDC_SIZENS));
		} else {
			SetCursor(LoadCursor(NULL, IDC_ARROW));
		}
		break;

	case WM_RBUTTONUP:
		SendMessage(hWnd, WM_CONTEXTMENU, (WPARAM)hWnd, lParam);
		break;

	case WM_PAINT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		hdc = BeginPaint(hWnd, &ps);

		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, ps.rcPaint.bottom,
			bf->draw_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);

		EndPaint(hWnd, &ps);
		break;

	case WM_ERASEBKGND:
		return TRUE;

	case WM_SCORE_DRAW_INIT:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_SCORE_REDRAW:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_score(hWnd, bf);
		InvalidateRect(hWnd, NULL, FALSE);
		UpdateWindow(hWnd);
		break;

	case WM_SCORE_REDRAW_DELAY:
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		SetTimer(hWnd, ID_DRAW_TIMER, TIMER_INTERVAL, NULL);
		break;

	case WM_TIMER:
		KillTimer(hWnd, wParam);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_CONTEXTMENU:
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			HMENU hMenu;
			POINT apos;
			DWORD st, en;
			WORD lang;
			int i;

			SendMessage(hWnd, EM_GETSEL, (WPARAM)&st, (LPARAM)&en);
			lang = PRIMARYLANGID(LANGIDFROMLCID(GetThreadLocale()));

			// j[̍쐬
			hMenu = CreatePopupMenu();
			if (op.full_screen != 0) {
				AppendMenu(hMenu, MF_STRING, 3,
					(lang != LANG_JAPANESE) ? TEXT("End &Fullscreen") : TEXT("tXN[(&F)"));
			} else {
				AppendMenu(hMenu, MF_STRING, 3,
					(lang != LANG_JAPANESE) ? TEXT("&Fullscreen") : TEXT("tXN[(&F)"));
			}
			if (op.visible_set != 0) {
				AppendMenu(hMenu, MF_STRING, 1,
					(lang != LANG_JAPANESE) ? TEXT("Hide &Sets") : TEXT("ZbgB(&S)"));
			} else {
				AppendMenu(hMenu, MF_STRING, 1,
					(lang != LANG_JAPANESE) ? TEXT("Show &Sets") : TEXT("Zbg\(&S)"));
			}
			AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
			AppendMenu(hMenu, MF_STRING, 2,
				(lang != LANG_JAPANESE) ? TEXT("&Close") : TEXT("(&C)"));

			// j[̕\
			GetCursorPos((LPPOINT)&apos);
			i = TrackPopupMenu(hMenu, TPM_TOPALIGN | TPM_RETURNCMD, apos.x, apos.y, 0, hWnd, NULL);
			DestroyMenu(hMenu);
			switch (i) {
			case 0:
				break;
			case 1:
				op.visible_set = (op.visible_set == 0) ? 1 : 0;
				draw_free(hWnd, bf);
				draw_init(hWnd, bf);
				draw_score(hWnd, bf);
				InvalidateRect(hWnd, NULL, FALSE);
				UpdateWindow(hWnd);
				break;
			case 2:
				SendMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			case 3:
				if (op.full_screen != 0) {
					op.full_screen = 0;
					ShowWindow(hWnd, SW_SHOWNORMAL);
				} else {
					op.full_screen = 1;
					ShowWindow(hWnd, SW_MAXIMIZE);
				}
				break;
			}
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * view_window_regist - EBhENX̓o^
 */
BOOL view_window_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)score_window_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * view_window_create - EBhE̍쐬
 */
HWND view_window_create(const HINSTANCE hInstance, SCORE_INFO *si)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindowEx(0,
		WINDOW_CLASS,
		WINDOW_TITLE,
		WS_POPUP,// | WS_THICKFRAME,
		op.window_rect.left,
		op.window_rect.top,
		op.window_rect.right,
		op.window_rect.bottom,
		NULL, NULL, hInstance, si);
	if (hWnd == NULL) {
		return NULL;
	}
	if (op.full_screen != 0) {
		ShowWindow(hWnd, SW_MAXIMIZE);
	} else {
		ShowWindow(hWnd, SW_SHOW);
	}
	return hWnd;
}
/* End of source */
